from django.shortcuts import render
from rest_framework import views, generics
from rest_framework.views import APIView
from rest_framework.response import Response
from rest_framework.renderers import JSONRenderer
from rest_framework.permissions import IsAuthenticated

from knox.auth import TokenAuthentication
from knox.models import AuthToken

from .models import *
from .serializers import *
from .utils.generators import GenerateTextHash

from random import randint
from datetime import datetime


class LoginPage(APIView):
    def get(self, request):
        return Response("Login page")


class LoginView(generics.GenericAPIView):
    serializer_class = UserLoginSerializer

    def post(self, request, *args, **kwargs):
        serializer = self.get_serializer(data=request.data)
        serializer.is_valid(raise_exception=True)
        user = serializer.validated_data

        return Response({
            "user": str(user),
            "token": AuthToken.objects.create(user)[1]
        })


class MainPage(APIView):
    authentication_classes = [TokenAuthentication]
    permission_classes = [IsAuthenticated]

    def get(self, request):
        user = request.user

        texts = Text.objects.filter(author=user)
        reviews = Review.objects.filter(author=user)

        texts_serializer = TextReadSerializer(texts, many=True)
        reviews_serializer = ReviewSerializer(reviews, many=True)

        response = {
            "texts": texts_serializer.data,
            "reviews": reviews_serializer.data
        }

        return Response(response)


class SettingsPage(APIView):
    authentication_classes = [TokenAuthentication]
    permission_classes = [IsAuthenticated]

    def post(self, request):
        return Response("Settings saving method")



# Text writing

class NewText(APIView):
    authentication_classes = [TokenAuthentication]
    permission_classes = [IsAuthenticated]

    def post(self, request):
        return Response("Return text writing page")


class GetTheme(APIView):
    authentication_classes = [TokenAuthentication]
    permission_classes = [IsAuthenticated]

    def post(self, request):
        user = request.user

        themes = Theme.objects.filter(genres__in=user.genres.all())
        if ("prev_theme" in request.data) and len(themes) > 1:
            themes.exclude(name=request.data["prev_theme"])
        theme_id = randint(1, len(themes)) - 1
        new_theme = themes[theme_id]

        genres = new_theme.genres.all()
        if ("prev_genre" in request.data) and len(themes) == 1 and len(genres) > 1:
            genres.exclude(name=request.data["prev_genre"])
        genre_id = randint(1, len(genres)) - 1
        new_genre = genres[genre_id]

        if new_theme == request.data["prev_theme"] and new_genre == request.data["prev_genre"]:
            return Response({
                "status": "error",
                "message": "Мы не смогли подобрать вам другой текст"
            })
        else:
            return Response({
                "status": "ok",
                "new_genre": str(new_genre.name),
                "new_theme": str(new_theme.name)
            })


class PublishText(APIView):
    authentication_classes = [TokenAuthentication]
    permission_classes = [IsAuthenticated]

    def post(self, request):
        user = request.user

        new_text = Text()
        new_text.content = request.data["content"]
        new_text.author = user
        new_text.creation_date = datetime.now()
        new_text.hash = GenerateTextHash()
        if "title" in request.data:
            new_text.title = request.data["title"]

        try:
            new_text.theme = Theme.objects.get(name=requset.data["theme"])
            new_text.genre = Genre.objects.get(name=request.data["genre"])
        except Theme.DoesNotExist or Genre.DoesNotExist:
            return Response({
                "status": "error",
                "message": "Жанр или тема неккоректны"
            })

        new_text.is_published = True
        new_text.save()
        return Response({
            "status": "ok"
        })



# Text reading

class GetText(APIView):
    authentication_classes = [TokenAuthentication]
    permission_classes = [IsAuthenticated]

    def post(self, request):
        user = request.user
        reviewed_textes = Review.objects.filter(author=user).values_list('text', flat=True)

        textes = Text.objects.filter(is_published=True).exclude(author=user)
        textes = textes.exclude(pk__in=reviewed_textes)
        if "prev_text" in request.data:
            textes = textes.exclude(hash=request.data["prev_text"])

        if len(textes) == 0:
            return Response({
                "status": "error",
                "message": "Очень жаль, но для вас не нашлось текста"
            })

        id = randint(1, len(textes)) - 1
        text_serializer = TextReadSerializer(textes[id])
        return Response(text_serializer.data)


class PublishReview(APIView):
    authentication_classes = [TokenAuthentication]
    permission_classes = [IsAuthenticated]

    def post(self, request):
        user = request.user

        new_review = Review()
        new_review.author = user
        new_review.text = Text.objects.get(hash=request.data["hash"])
        new_review.creation_date = datetime.now()
        new_review.summary = request.data["summary"]
        new_review.mark = request.data["mark"]
        new_review.is_published = True
        new_review.save()

        review_segs = request.data["review_segments"]
        for i in review_segs:
            new_review_segment = ReviewSegment()
            new_review_segment.review = new_review
            new_review_segment.position = i.position
            new_review_segment.comment = i.comment
            new_review_segment.length = i.length
            new_review_segment.save()

        return Response({ "status": "ok" })


class ReadText(APIView):
    authentication_classes = [TokenAuthentication]
    permission_classes = [IsAuthenticated]

    def get(self, request, hash):
        user = request.user
        try:
            text = Text.objects.get(hash=hash)
            response = {}
            if text.author == user:
                response = TextFullSerializer(text).data
            else:
                response = TextReadSerializer(text).data
                reviews = Review.objects.filter(text=text.pk, author=user)
                if len(reviews) == 1:
                    response["review"] = ReviewSerializer(reviews[0]).data

            response["theme"] = Theme.objects.get(pk=response["theme"]).name
            response["genre"] = Genre.objects.get(pk=response["genre"]).name

            return Response(response)
        except Text.DoesNotExist:
            return Response({
                "status": "error",
                "message": "Текст не существует"
            })
